#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-08-29
#
# inventory of bgp peers
#
# 2022-04-30: code cleanup/streamlining
# 2023-01-17: always remove fsm_established_time
# 2025-01-17: fixed non working whois retrieval (param check was incorrect, no default rir is also not True)
#             fixed crash on missing remote_as
#             fixed missing "/" in whois query (ThX to nick[at]fisk[dot]me[dot]uk)

import time
from typing import List

from cmk.agent_based.v2 import (
    InventoryPlugin,
    InventoryResult,
    OIDBytes,
    SNMPSection,
    SNMPTree,
    StringByteTable,
    TableRow,
    exists,
)

from cmk_addons.plugins.bgp_peer.lib.bgp_peer import (
    BgpWhois,
    InvBgpPeer,
    bgp_error_as_string,
    bgp_error_code_as_hex,
    get_bgp_type,
)


def parse_inv_bgp_peer(string_table: List[StringByteTable]):
    peers, base = string_table
    try:
        local_as, local_id = base[0]
    except (ValueError, IndexError):
        local_id = 'N/A'
        local_as = 'N/A'

    bgp_peers = []

    for entry in peers:
        try:
            remote_id,  state, version, local_addr, remote_addr, remote_as, last_error, fsm_established_time = entry
        except ValueError:
            return

        bgp_peer: InvBgpPeer = {
            'remote_addr': remote_addr,
            'remote_id': remote_id,
            'version': version,
            'local_addr': local_addr,
            'remote_as': remote_as,
            'local_as': local_as,
            'local_id': local_id,
            'bgp_type': get_bgp_type(local_as, remote_as),
            'fsm_established_time': int(fsm_established_time) if fsm_established_time.isdigit() else None,
            'peer_state': 1 if state == '6' else 2,  # adjust to match if_oper_status for inventory painter
            'last_error_code': bgp_error_code_as_hex(last_error),
            'last_error': bgp_error_as_string(last_error),
            'status_columns': {},
            'address_family': 'N/A'
        }

        bgp_peers.append(bgp_peer)

    return bgp_peers


def inventory_bgp_peers(params, section) -> InventoryResult:
    path = ['networking', 'bgp_peers']

    if (whois := params.get('whois_enable')) is not None:
        whois = BgpWhois(
            default_rir=params['whois_enable'].get('whois_rir', 'https://rdap.db.ripe.net/'),
            timeout=params['whois_enable'].get('whois_timeout', 5)
        )

    for bgp_peer in section:
        key_columns = {'remote_addr': bgp_peer['remote_addr']}

        for key in key_columns.keys():
            bgp_peer.pop(key)

        status_columns = bgp_peer['status_columns']
        bgp_peer.pop('status_columns')

        if whois and bgp_peer.get('remote_as', '').isdigit():
            as_info = whois.get_whois_data_by_asn(int(bgp_peer.get('remote_as')))
            bgp_peer.update(as_info)

        for column in params.get('remove_columns', []):
            try:
                bgp_peer.pop(column)
            except KeyError:
                pass

        fsm_established_time = bgp_peer.get('fsm_established_time')
        try:
            bgp_peer.pop('fsm_established_time')
        except KeyError:
            pass

        if fsm_established_time:
            peer_state = bgp_peer.get('peer_state')
            in_service = True
            not_in_service_time = params.get('not_in_service_time', 2592000)
            if peer_state == 2:  # not established
                if fsm_established_time >= not_in_service_time:
                    in_service = False

            bgp_peer.update({'in_service': in_service})
            bgp_peer.update({'last_change': time.time() - fsm_established_time})

        yield TableRow(
            path=path,
            key_columns=key_columns,
            inventory_columns=bgp_peer,
            status_columns=status_columns
        )

    if whois:
        del whois


snmp_section_inv_bgp_peer = SNMPSection(
    name='inv_bgp_peer',
    parse_function=parse_inv_bgp_peer,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.2.1.15.3.1',  # BGP4-MIB::BgpPeerEntry
            oids=[
                '1',  # bgpPeerIdentifier
                '2',  # bgpPeerState
                '4',  # bgpPeerNegotiatedVersion
                '5',  # bgpPeerLocalAddr
                '7',  # bgpPeerRemoteAddr
                '9',  # bgpPeerRemoteAs
                OIDBytes('14'),  # bgpPeerLastError
                '16',  # bgpPeerFsmEstablishedTime
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.2.1.15',  # BGP4-MIB
            oids=[
                '2',  # bgpLocalAs
                '4',  # bgpIdentifier (local)
            ]
        )
    ],
    detect=exists('.1.3.6.1.2.1.15.3.1.1.*')
)

inventory_plugin_inv_bgp_peer = InventoryPlugin(
    name='inv_bgp_peer',
    inventory_function=inventory_bgp_peers,
    inventory_default_parameters={
        'not_in_service_time': 2592000,  # 30 days in seconds
    },
    inventory_ruleset_name='inv_bgp_peer',
)
