#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-08-21
#
# BGP Peer metrics plugin
#
# 2024-05-17: fixed typo in range mac -> max (crashed graphs in cmk 2.3.)
# 2025-03-29: rewritten for graphing API v1

from cmk.graphing.v1 import Title, graphs, metrics, perfometers


UNIT_COUNTER = metrics.Unit(metrics.DecimalNotation(''), metrics.StrictPrecision(2))
UNIT_PER_SECOND = metrics.Unit(metrics.DecimalNotation('/s'))
UNIT_TIME = metrics.Unit(metrics.TimeNotation())

metric_bgp_peer_accepted_prefixes = metrics.Metric(
    name='bgp_peer_acceptedprefixes',
    title=Title("Prefixes accepted"),
    unit=UNIT_COUNTER,
    color=metrics.Color.DARK_PURPLE,
)

metric_bgp_peer_advertised_prefixes = metrics.Metric(
    name='bgp_peer_advertisedprefixes',
    title=Title("Prefixes advertised"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.CYAN,
)

metric_bgp_peer_denied_prefixes = metrics.Metric(
    name='bgp_peer_deniedprefixes',
    title=Title("Prefixes denied"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.YELLOW,
)

metric_bgp_peer_fsm_established_time = metrics.Metric(
    name='bgp_peer_fsmestablishedtime',
    title=Title("FSM last change"),
    unit=UNIT_TIME,
    color=metrics.Color.GREEN,
)

metric_bgp_peer_fsm_established_transitions = metrics.Metric(
    name='bgp_peer_fsmestablishedtransitions',
    title=Title("FSM transitions"),
    unit=UNIT_COUNTER,
    color=metrics.Color.YELLOW,
)

metric_bgp_peer_in_total_messages = metrics.Metric(
    name='bgp_peer_intotalmessages',
    title=Title("Messages received"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.DARK_BLUE,
)

metric_bgp_peer_in_update_elapsed_time = metrics.Metric(
    name='bgp_peer_inupdateelapsedtime',
    title=Title("Last update received"),
    unit=UNIT_TIME,
    color=metrics.Color.DARK_BLUE,
)

metric_bgp_peer_in_updates = metrics.Metric(
    name='bgp_peer_inupdates',
    title=Title("Updates received"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.YELLOW,
)

metric_bgp_peer_out_total_messages = metrics.Metric(
    name='bgp_peer_outtotalmessages',
    title=Title("Messages send"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.DARK_PINK,
)

metric_bgp_peer_out_updates = metrics.Metric(
    name='bgp_peer_outupdates',
    title=Title("Updates send"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.CYAN,
)

metric_bgp_peer_suppressed_prefixes = metrics.Metric(
    name='bgp_peer_suppressedprefixes',
    title=Title("Prefixes suppressed"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.DARK_PINK,
)

metric_bgp_peer_withdrawn_prefixes = metrics.Metric(
    name='bgp_peer_withdrawnprefixes',
    title=Title("Prefixes withdrawn"),
    unit=UNIT_PER_SECOND,
    color=metrics.Color.BLUE,
)

# Juniper specific metrics
metric_bgp_peer_in_prefixes = metrics.Metric(
    name='bgp_peer_in_prefixes',
    title=Title("Prefixes in"),
    unit=UNIT_COUNTER,
    color=metrics.Color.DARK_PURPLE,
)

metric_bgp_peer_in_prefixes_rejected = metrics.Metric(
    name='bgp_peer_in_prefixes_rejected',
    title=Title("Prefixes in rejected"),
    unit=UNIT_COUNTER,
    color=metrics.Color.YELLOW,
)

metric_bgp_peer_in_prefixes_active = metrics.Metric(
    name='bgp_peer_in_prefixes_active',
    title=Title("Prefixes in active"),
    unit=UNIT_COUNTER,
    color=metrics.Color.CYAN,
)

metric_bgp_peer_out_prefixes = metrics.Metric(
    name='bgp_peer_out_prefixes',
    title=Title("Prefixes out"),
    unit=UNIT_COUNTER,
    color=metrics.Color.BLUE,
)

# Huawei specific metrics
metric_bgp_peer_prefix_adv_counter = metrics.Metric(
    name='bgp_peer_prefixadvcounter',
    title=Title("Prefixes advertised"),
    unit=UNIT_COUNTER,
    color=metrics.Color.DARK_BLUE,
)

metric_bgp_peer_prefix_active_counter = metrics.Metric(
    name='bgp_peer_prefixactivecounter',
    title=Title("Prefixes active"),
    unit=UNIT_COUNTER,
    color=metrics.Color.CYAN,
)

metric_bgp_peer_prefix_rcv_counter = metrics.Metric(
    name='bgp_peer_prefixrcvcounter',
    title=Title("Prefixes received"),
    unit=UNIT_COUNTER,
    color=metrics.Color.DARK_PURPLE,
)

graph_bgp_peer_fms_transitions_last_change = graphs.Graph(
    name='bgp_peer.fms_transitions_last_change',
    title=Title("FSM established last change"),
    minimal_range=graphs.MinimalRange(0, metrics.MaximumOf('bgp_peer_fsmestablishedtime', metrics.Color.GRAY, ), ),
    compound_lines=['bgp_peer_fsmestablishedtime'],
)

graph_bgp_peer_prefixes_accepted = graphs.Graph(
    name='bgp_peer.prefixes_accepted',
    title=Title("Accepted Prefixes"),
    minimal_range=graphs.MinimalRange(0, metrics.MaximumOf('bgp_peer_acceptedprefixes', metrics.Color.GRAY, ), ),
    compound_lines=['bgp_peer_acceptedprefixes'],
    simple_lines=[
        metrics.CriticalOf('bgp_peer_acceptedprefixes'),
        metrics.WarningOf('bgp_peer_acceptedprefixes'),
    ],
)

graph_bgp_peer_prefixes_per_second = graphs.Graph(
    name='bgp_peer.prefixes_per_second',
    title=Title("Prefixes/s"),
    simple_lines=[
        'bgp_peer_withdrawnprefixes',
        'bgp_peer_suppressedprefixes',
        'bgp_peer_deniedprefixes',
        'bgp_peer_advertisedprefixes',
    ],
    optional=[
        'bgp_peer_withdrawnprefixes',
        'bgp_peer_suppressedprefixes',
        'bgp_peer_deniedprefixes',
        'bgp_peer_advertisedprefixes',
    ],
)

graph_bgp_peer_updates_in_out = graphs.Bidirectional(
    name='bgp_peer.updates_in_out',
    title=Title("Updates"),
    lower=graphs.Graph(
        name='bgp_peer.updates_in_out_lower',
        title=Title("Updates"),
        compound_lines=['bgp_peer_outupdates'],
    ),
    upper=graphs.Graph(
        name='bgp_peer.updates_in_out_upper',
        title=Title("Updates"),
        compound_lines=['bgp_peer_inupdates'],
    ),
)

graph_bgp_peer_messages_in_out = graphs.Bidirectional(
    name='bgp_peer.messages_in_out',
    title=Title("Total messages"),
    lower=graphs.Graph(
        name='bgp_peer.messages_in_out_lower',
        title=Title("Total messages"),
        compound_lines=['bgp_peer_outtotalmessages'],
    ),
    upper=graphs.Graph(
        name='bgp_peer.messages_in_out_upper',
        title=Title("Total messages"),
        compound_lines=['bgp_peer_intotalmessages'],
    ),
)

graph_bgp_peer_fms_transitions_from_to = graphs.Graph(
    name='bgp_peer.fms_transitions_from_to',
    title=Title("FSM transitions from/to established"),
    minimal_range=graphs.MinimalRange(
        0, metrics.MaximumOf('bgp_peer_fsmestablishedtransitions', metrics.Color.GRAY, ),
    ),
    compound_lines=['bgp_peer_fsmestablishedtransitions'],
)

graph_bgp_peer_time_since_last_update = graphs.Graph(
    name='bgp_peer.time_since_last_update',
    title=Title("Time since last update received"),
    minimal_range=graphs.MinimalRange(
        0, metrics.MaximumOf('bgp_peer_inupdateelapsedtime', metrics.Color.GRAY, ),
    ),
    compound_lines=['bgp_peer_inupdateelapsedtime'],
)

# juniper prefixes
graph_bgp_peer_juniper_prefixes = graphs.Bidirectional(
    name='bgp_peer.juniper_prefixes',
    title=Title("Prefixes in/out"),
    lower=graphs.Graph(
        name='bgp_peer.juniper_prefixes_lower',
        title=Title("Prefixes in/out"),
        simple_lines=['bgp_peer_out_prefixes'],
    ),
    upper=graphs.Graph(
        name='bgp_peer.juniper_prefixes_upper',
        title=Title("Prefixes in/out"),
        simple_lines=[
            'bgp_peer_in_prefixes_rejected',
            'bgp_peer_in_prefixes_active',
            'bgp_peer_in_prefixes',
        ],
    ),
)

# huawei prefixes
graph_huawei_bgp_peer_counter = graphs.Graph(
    name='huawei_bgp_peer_counter',
    title=Title("BGP prefix counter"),
    simple_lines=[
        'bgp_peer_prefixrcvcounter',
        'bgp_peer_prefixactivecounter',
        'bgp_peer_prefixadvcounter',
    ],
)


perfometer_bgp_peer_accepted_prefixes = perfometers.Stacked(
    name='bgp_peer_stacked',
    upper=perfometers.Perfometer(
        name='bgp_peer_fsmestablishedtime',
        segments=['bgp_peer_fsmestablishedtime'],
        focus_range=perfometers.FocusRange(perfometers.Closed(0), perfometers.Open(2592000)) # ome month
    ),
    lower=perfometers.Perfometer(
        name='bgp_peer_acceptedprefixes',
        segments=['bgp_peer_acceptedprefixes'],
        focus_range=perfometers.FocusRange(perfometers.Closed(0), perfometers.Open(500000))
    )
)

perfometer_bgp_peer_fsm_established_time = perfometers.Perfometer(
    name='bgp_peer_fsmestablishedtime',
    segments=['bgp_peer_fsmestablishedtime'],
    focus_range=perfometers.FocusRange(perfometers.Closed(0), perfometers.Open(2592000)) # ome month
)